aboutsummaryrefslogtreecommitdiffstats
path: root/eth
diff options
context:
space:
mode:
authorPéter Szilágyi <peterke@gmail.com>2016-07-26 18:07:12 +0800
committerPéter Szilágyi <peterke@gmail.com>2016-07-26 18:07:12 +0800
commit8f0a4a25f82f48005e6252a90c008bdc76219cc3 (patch)
tree3df0addc590cbb368c0ab24b7993caf9eafaaac4 /eth
parenta724952f7530d6709abada4d4da727409f703eb3 (diff)
downloaddexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.tar
dexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.tar.gz
dexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.tar.bz2
dexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.tar.lz
dexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.tar.xz
dexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.tar.zst
dexon-8f0a4a25f82f48005e6252a90c008bdc76219cc3.zip
eth/downloader: abort sync if master drops (timeout prev)
Diffstat (limited to 'eth')
-rw-r--r--eth/downloader/downloader.go17
1 files changed, 15 insertions, 2 deletions
diff --git a/eth/downloader/downloader.go b/eth/downloader/downloader.go
index aee21122a..79e2ea2c0 100644
--- a/eth/downloader/downloader.go
+++ b/eth/downloader/downloader.go
@@ -147,8 +147,10 @@ type Downloader struct {
stateWakeCh chan bool // [eth/63] Channel to signal the state fetcher of new tasks
headerProcCh chan []*types.Header // [eth/62] Channel to feed the header processor new tasks
+ // Cancellation and termination
+ cancelPeer string // Identifier of the peer currently being used as the master (cancel on drop)
cancelCh chan struct{} // Channel to cancel mid-flight syncs
- cancelLock sync.RWMutex // Lock to protect the cancel channel in delivers
+ cancelLock sync.RWMutex // Lock to protect the cancel channel and peer in delivers
quitCh chan struct{} // Quit channel to signal termination
quitLock sync.RWMutex // Lock to prevent double closes
@@ -254,12 +256,22 @@ func (d *Downloader) RegisterPeer(id string, version int, head common.Hash,
// the specified peer. An effort is also made to return any pending fetches into
// the queue.
func (d *Downloader) UnregisterPeer(id string) error {
+ // Unregister the peer from the active peer set and revoke any fetch tasks
glog.V(logger.Detail).Infoln("Unregistering peer", id)
if err := d.peers.Unregister(id); err != nil {
glog.V(logger.Error).Infoln("Unregister failed:", err)
return err
}
d.queue.Revoke(id)
+
+ // If this peer was the master peer, abort sync immediately
+ d.cancelLock.RLock()
+ master := id == d.cancelPeer
+ d.cancelLock.RUnlock()
+
+ if master {
+ d.cancel()
+ }
return nil
}
@@ -332,9 +344,10 @@ func (d *Downloader) synchronise(id string, hash common.Hash, td *big.Int, mode
empty = true
}
}
// Create cancel channel for aborting mid-flight
d.cancelLock.Lock()
d.cancelCh = make(chan struct{})
+ d.cancelPeer = id
d.cancelLock.Unlock()
defer d.cancel() // No matter what, we can't leave the cancel channel open
@@ -332,9 +344,10 @@ func (d *Downloader) synchronise(id string, hash common.Hash, td *big.Int, mode
}
}
- // Create cancel channel for aborting mid-flight
+ // Create cancel channel for aborting mid-flight and mark the master peer
d.cancelLock.Lock()
d.cancelCh = make(chan struct{})